@@ -6,5 +6,5 @@ import GHC.Prelude
 
 #define MACHREGS_NO_REGS 0
 #define MACHREGS_x86_64 1
-#include "../../../includes/CodeGen.Platform.hs"
+#include "CodeGen.Platform.h"
 
